'use client';
import { jsx } from 'react/jsx-runtime';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import { Collapse, useTheme } from '@mui/material';
import { useContext, createContext, useState, useCallback, useMemo, useRef, Children, isValidElement, cloneElement } from 'react';

const CollapsibleContext = /*#__PURE__*/ createContext(null);
const useCollapsible = ()=>{
    const context = useContext(CollapsibleContext);
    if (!context) {
        throw new Error('useCollapsible must be used within a CollapsibleProvider.');
    }
    return context;
};
/**
 * `Collapsible` 一個可以透過觸發按鈕展開或收起區塊，跟 `Accordion` 的差別在於 `Collapsible` 可以自由控制開關。
 *
 * ref: https://mui.com/material-ui/api/collapse/
 *
 * @example
 *
 * ```tsx
 * <Collapsible>
 *  <CollapsibleTrigger>
 *    <Button>Toggle</Button>
 *  </CollapsibleTrigger>
 *  <CollapsibleContent>
 *    <div>Content</div>
 *  </CollapsibleContent>
 * </Collapsible>
 * ```
 */ const Collapsible = ({ children, defaultOpen = true, open: outerOpen, onOpenChange: setOuterOpen })=>{
    // This is the internal state of the Collapsible.
    // We use outerOpen and setOuterOpen for control from outside the component.
    const [innerOpen, setInnerOpen] = useState(defaultOpen);
    const open = outerOpen ?? innerOpen;
    const setOpen = useCallback((value)=>{
        const openState = typeof value === 'function' ? value(open) : value;
        if (setOuterOpen) {
            setOuterOpen(openState);
        } else {
            setInnerOpen(openState);
        }
    }, [
        setOuterOpen,
        open
    ]);
    // Helper to toggle the Collapsible.
    const toggleCollapsible = useCallback(()=>setOpen((preOpen)=>!preOpen), [
        setOpen
    ]);
    const contextValue = useMemo(()=>({
            open,
            setOpen,
            toggleCollapsible
        }), [
        open,
        setOpen,
        toggleCollapsible
    ]);
    return /*#__PURE__*/ jsx(CollapsibleContext.Provider, {
        value: contextValue,
        children: children
    });
};
const CollapsibleTrigger = ({ children })=>{
    const anchorRef = useRef(null);
    const element = Children.only(children);
    const { toggleCollapsible } = useCollapsible();
    if (/*#__PURE__*/ isValidElement(children)) {
        return /*#__PURE__*/ cloneElement(element, {
            ref: anchorRef,
            onClick: ()=>{
                element.props.onClick?.();
                toggleCollapsible();
            }
        });
    }
    return Children.count(children) > 1 ? Children.only(null) : null;
};
const CollapsibleContent = ({ children })=>{
    const { open } = useCollapsible();
    return /*#__PURE__*/ jsx(Collapse, {
        in: open,
        children: children
    });
};
/**
 * An icon that rotates when the Collapsible is open.
 */ const CollapsibleIcon = ({ sx, ...props })=>{
    const theme = useTheme();
    const { open } = useCollapsible();
    return /*#__PURE__*/ jsx(ExpandMoreIcon, {
        sx: {
            transition: theme.transitions.create('transform'),
            transform: open ? 'rotate(180deg)' : 'rotate(0deg)',
            ...sx
        },
        ...props
    });
};

export { Collapsible as C, CollapsibleTrigger as a, CollapsibleContent as b, CollapsibleIcon as c, useCollapsible as u };
